@@ -28,11 +28,6 @@
 #include "pdns/logger.hh"
 #include "pdns/arguments.hh"
 
-/* 
-    virtual void getUpdatedMasters(vector<DomainInfo>* domains);
-    virtual void setNotifed(int id, uint32_t serial);
-*/
-
 void LUABackend::getUpdatedMasters(vector<DomainInfo>* domains) {
 	
     if (f_lua_getupdatedmasters == 0)
@@ -63,15 +58,15 @@ void LUABackend::getUpdatedMasters(vector<DomainInfo>* domains) {
 	L << Logger::Info << backend_name << "(getUpdatedMasters) END" << endl;
 }
 
-void LUABackend::setNotifed(int id, uint32_t serial) {
+void LUABackend::setNotified(uint32_t id, uint32_t serial) {
 	
-    if (f_lua_setnotifed == 0)
+    if (f_lua_setnotified == 0)
 	return;
 
     if (logging)
-	L << Logger::Info << backend_name << "(setNotifed) BEGIN" << endl;
+	L << Logger::Info << backend_name << "(setNotified) BEGIN" << endl;
 
-    lua_rawgeti(lua, LUA_REGISTRYINDEX, f_lua_setnotifed);
+    lua_rawgeti(lua, LUA_REGISTRYINDEX, f_lua_setnotified);
 
     lua_pushinteger(lua, id);
     lua_pushinteger(lua, serial);
@@ -85,6 +80,6 @@ void LUABackend::setNotifed(int id, uint32_t serial) {
     }
 
     if (logging)
-	L << Logger::Info << backend_name << "(setNotifed) END" << endl;
+	L << Logger::Info << backend_name << "(setNotified) END" << endl;
 }
 
